package org.basex.client
import java.net.HttpURLConnection
import java.net.URL
import scala.io.Source
import java.io.BufferedOutputStream
import java.io.InputStream
import java.io.OutputStream

/**
 * This class contains example methods to communicate with BaseX' JAX-RX REST API.
 *
 * @author Lukas Lewandowski, University of Konstanz, Germany.
 *
 */
class JaxRxClient {

  /** Server and JAX-RX address. */
  val srv = "http://localhost:8984/basex/jax-rx"
  /** Resource example. */
  val res = "myresource"
  /** Resource XML example. */
  val xml = "<names><name>Steve</name><name>William</name>" +
    "<name>Mark</name><name>Larry</name><name>Sergey</name>" +
    "<name>Christian</name></names>"
  /** Second XML example. */
  val xml2 = "<names><name>Tom</name><name>Jerry</name></names>"

  /** Example query. Returns all available names in document. */
  val query = "/names/name/text()"
  /** Example command. */
  val command = "info"
  /** UTF-8 encoding. */
  val utf8 = "UTF-8"
  /** Content-Type String. */
  val ct = "Content-Type"
  /** Console print message 1. */
  val p1 = "Http response code: "
  /** Console print message 2. */
  val p2 = "Result:"
  /** Content type application/query+xml */
  val qct = "application/query+xml"
  /** HTTP POST String. */
  val post = "POST"
    /** HTTP PUT String. */
  val put = "PUT"

  /**
   * HTTP GET example. Returns all available documents and collections.
   */
  def get {
    val url = new URL(srv)
    val conn = url.openConnection.asInstanceOf[HttpURLConnection]
    conn.connect
    val code = conn.getResponseCode
    println(p1 + code)
    if (code == HttpURLConnection.HTTP_OK)
      printToConsole(conn.getInputStream())
    conn.disconnect
  }

  /**
   * HTTP GET example. Returns all available documents and collections.
   */
  def getResource {
    val url = new URL(srv + "/" + res)
    val conn = url.openConnection.asInstanceOf[HttpURLConnection]
    conn.connect
    val code = conn.getResponseCode
    println(p1 + code)
    if (code == HttpURLConnection.HTTP_OK)
      printToConsole(conn.getInputStream())
    conn.disconnect
  }

  /** Deletes an existing resource. */
  def delete {
    val url = new URL(srv + "/" + res)
    val conn = url.openConnection.asInstanceOf[HttpURLConnection]
    conn.setRequestMethod("DELETE")
    conn.connect
    val code = conn.getResponseCode
    println(p1 + code)
    if (code == HttpURLConnection.HTTP_OK)
      printToConsole(conn.getInputStream())
    conn.disconnect

  }

  /**
   * HTTP GET example. Returns a query result.
   */
  def queryByGet {
    val url = new URL(srv + "/" + "xmark" + "?query=" + ".")
    val conn = url.openConnection.asInstanceOf[HttpURLConnection]
    conn.connect
    val code = conn.getResponseCode
    println(p1 + code)
    if (code == HttpURLConnection.HTTP_OK)
      printToConsole(conn.getInputStream())
    conn.disconnect

  }

  /**
   * HTTP POST example. Returns a query result.
   */
  def queryByPost {
    val cont = "<query><text>" + query + "</text></query>";
    val url = new URL(srv + "/" + res)
    val conn = url.openConnection.asInstanceOf[HttpURLConnection]
    conn.setRequestMethod(post)
    conn.setRequestProperty(ct, qct)
    conn.setDoOutput(true)
    writeToOutput(conn.getOutputStream(), cont)
    conn.connect
    val code = conn.getResponseCode
    println(p1 + code)
    if (code == HttpURLConnection.HTTP_OK)
      printToConsole(conn.getInputStream())
    conn.disconnect

  }

  /** Creates a new resource or updates an existing resource. */
  def update {
    val url = new URL(srv + "/" + res)
    val conn = url.openConnection.asInstanceOf[HttpURLConnection]
    conn.setRequestMethod(put)
    conn.setRequestProperty(ct, "text/xml")
    conn.setDoOutput(true)
    writeToOutput(conn.getOutputStream(), xml)
    conn.connect
    val code = conn.getResponseCode
    println(p1 + code)
    if (code == HttpURLConnection.HTTP_CREATED)
      printToConsole(conn.getInputStream())
    conn.disconnect

  }

  /** Adds a document to an existing resource. */
  def add {
    val url = new URL(srv + "/" + res)
    val conn = url.openConnection.asInstanceOf[HttpURLConnection]
    conn.setRequestMethod(post)
    conn.setRequestProperty(ct, "application/xml")
    conn.setDoOutput(true)
    writeToOutput(conn.getOutputStream(), xml2)
    conn.connect
    val code = conn.getResponseCode
    println(p1 + code)
    if (code == HttpURLConnection.HTTP_CREATED)
      printToConsole(conn.getInputStream())
    conn.disconnect
  }

  /**
   *  Runs an XQuery file existing on server. The file receives as
   *  external variable We want the second name item value from our example XML
   */
  def runByPost {
    val url = new URL(srv)
    val cont = "<run><text>getname.xq</text><variable name='item' value='2'/></run>";
    val conn = url.openConnection.asInstanceOf[HttpURLConnection]
    conn.setRequestMethod(post)
    conn.setRequestProperty(ct, qct)
    conn.setDoOutput(true)
    writeToOutput(conn.getOutputStream(), cont)
    conn.connect
    val code = conn.getResponseCode
    println(p1 + code)
    if (code == HttpURLConnection.HTTP_OK)
      printToConsole(conn.getInputStream())
    conn.disconnect
  }

  /**
   *  Executes a command by POST request. It is also
   *  possible to use GET as in queryByGet
   */
  def commandByPost {
    val url = new URL(srv + "/" + res)
    val cont = "<command><text>" + command + "</text></command>";
    val conn = url.openConnection.asInstanceOf[HttpURLConnection]
    conn.setRequestMethod(post)
    conn.setRequestProperty(ct, qct)
    conn.setDoOutput(true)
    writeToOutput(conn.getOutputStream(), cont)
    conn.connect
    val code = conn.getResponseCode
    println(p1 + code)
    if (code == HttpURLConnection.HTTP_OK)
      printToConsole(conn.getInputStream())
    conn.disconnect
  }

  /**
   * This method prints the result to the console.
   *
   * @param is The InputStream received by the response.
   */
  private def printToConsole(is: InputStream) {
    println(p2)
    var line = ""
    val source = Source.fromInputStream(is, utf8)
    source.getLines().foreach((line) => {
      println(line)
    })
    source.close
  }

  /**
   * This method writes a String content to the OutputStream.
   * 
   * @param os The OutStream from the request object.
   * @param content The content to be written to the OutputStream.
   */
  private def writeToOutput(os: OutputStream, content: String) {
    val bos = new BufferedOutputStream(os)
    bos.write(content.getBytes())
    bos.close
  }

}